use super::ActionRunner;

pub(crate) struct InsertText;
pub(crate) struct InsertFromPaste;
pub(crate) struct InsertLineBreak;

const NL_BYTES: [u16; 1] = [10];

fn insert(start: usize, end: usize, data: Vec<u16>, mut doc: Vec<u16>) -> Vec<u16> {
    if doc.len() < start || data.is_empty() {
        return doc;
    }

    if start < end {
        doc.drain(start..end);
    }

    doc.splice(start..start, data);
    doc
}

impl ActionRunner for InsertText {
    fn run(&self, start: usize, end: usize, data: Vec<u16>, doc: Vec<u16>) -> Vec<u16> {
        insert(start, end, data, doc)
    }
}

impl ActionRunner for InsertFromPaste {
    fn run(&self, start: usize, end: usize, data: Vec<u16>, doc: Vec<u16>) -> Vec<u16> {
        insert(start, end, data, doc)
    }
}

impl ActionRunner for InsertLineBreak {
    fn run(&self, start: usize, _end: usize, _data: Vec<u16>, doc: Vec<u16>) -> Vec<u16> {
        insert(start, start, Vec::from(NL_BYTES), doc)
    }
}
